<?php
declare(strict_types=1);

namespace Inti\Wp\Plugin;

use _WP_Dependency;
use WP_Dependencies;
use function add_action;
use function home_url;

/**
 * Class Assets
 *
 * @package Inti\Wp\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class Assets implements PluginInterface
{
    private array $manifest;
    private array $dequeue;

    public function __construct(array $manifest, array $dequeue)
    {
        $this->manifest = $manifest;
        $this->dequeue = $dequeue;
    }

    public function run(?array $options = null): void
    {
        add_action('wp_print_footer_scripts', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_print_scripts', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_print_styles', [$this, 'wp_dequeue_scripts'], 0);
        add_action('wp_enqueue_scripts', [$this, 'fix_dependency_urls'], PHP_INT_MAX);
    }

    public function wp_dequeue_scripts()
    {
        global $wp_scripts, $wp_styles;

        if ($wp_scripts instanceof WP_Dependencies) {
            $wp_scripts->dequeue($this->dequeue['scripts'] ?? []);
        }

        if ($wp_styles instanceof WP_Dependencies) {
            $wp_styles->dequeue($this->dequeue['styles'] ?? []);
        }
    }

    public function fix_dependency_urls()
    {
        global $wp_scripts, $wp_styles;

        foreach ([$wp_scripts, $wp_styles] as $dependencies) {
            foreach ($dependencies->registered as $dependency) {
                /** @var _WP_Dependency $dependency */
                if (isset($this->manifest[$dependency->src])) {
                    $dependency->src = home_url($this->manifest[$dependency->src]);
                }
            }
        }
    }
}
